%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: ElementInfo.m
%==========================================================

function [str, ele] = ElementInfo(str, elements)
%------------------------------------------------
str.nele = size(elements,1);
for iele = 1:str.nele
    eletag  = elements{iele}{1};
    eletype = elements{iele}{2};
    eleinfo = elements{iele};
    ele(eletag).type = eletype;
    switch eletype
        case 'Simple2DTruss'
            [tmpeleinfo] = Simple2DTrussInfo(eleinfo);
        case 'Simple3DTruss'
            [tmpeleinfo] = Simple3DTrussInfo(eleinfo);
        case 'StiffnessBasedBeam'
            [tmpeleinfo] = StiffnessBasedBeamInfo(eleinfo);
        case 'StiffnessBased2DBeamColumn'
            [tmpeleinfo] = StiffnessBased2DBeamColumnInfo(eleinfo);
        case 'StiffnessBased3DBeamColumn'
            [tmpeleinfo] = StiffnessBased3DBeamColumnInfo(eleinfo);
        case 'Grid'
            [tmpeleinfo] = GridInfo(eleinfo);
    end
    ele(eletag).(eletype) = tmpeleinfo;
end
%------------------------------------------------
str.ndofpe = 2*str.ndofpn;
%------------------------------------------------
% Make LM matrix
if (str.ndim == 2 && str.ndofpn == 2)
    for iele = 1:str.nele
        eletype = ele(iele).type;
        str.LM(iele,1) = str.ID(ele(iele).(eletype).snode,1);
        str.LM(iele,2) = str.ID(ele(iele).(eletype).snode,2);
        str.LM(iele,3) = str.ID(ele(iele).(eletype).enode,1);
        str.LM(iele,4) = str.ID(ele(iele).(eletype).enode,2);
    end
elseif ( str.ndim == 2 && str.ndofpn == 3 || str.ndim == 3 && str.ndofpn == 3 )
    for iele = 1:str.nele
        eletype = ele(iele).type;
        str.LM(iele,1) = str.ID(ele(iele).(eletype).snode,1);
        str.LM(iele,2) = str.ID(ele(iele).(eletype).snode,2);
        str.LM(iele,3) = str.ID(ele(iele).(eletype).snode,3);
        str.LM(iele,4) = str.ID(ele(iele).(eletype).enode,1);
        str.LM(iele,5) = str.ID(ele(iele).(eletype).enode,2);
        str.LM(iele,6) = str.ID(ele(iele).(eletype).enode,3);
    end
elseif ( str.ndim == 3 && str.ndofpn == 6 )
    for iele = 1:ele.nele
        eletype = ele(iele).type;
        str.LM(iele, 1) = str.ID(ele(iele).(eletype).snode,1);
        str.LM(iele, 2) = str.ID(ele(iele).(eletype).snode,2);
        str.LM(iele, 3) = str.ID(ele(iele).(eletype).snode,3);
        str.LM(iele, 4) = str.ID(ele(iele).(eletype).snode,4);
        str.LM(iele, 5) = str.ID(ele(iele).(eletype).snode,5);
        str.LM(iele, 6) = str.ID(ele(iele).(eletype).snode,6);
        str.LM(iele, 7) = str.ID(ele(iele).(eletype).enode,1);
        str.LM(iele, 8) = str.ID(ele(iele).(eletype).enode,2);
        str.LM(iele, 9) = str.ID(ele(iele).(eletype).enode,3);
        str.LM(iele,10) = str.ID(ele(iele).(eletype).enode,4);
        str.LM(iele,11) = str.ID(ele(iele).(eletype).enode,5);
        str.LM(iele,12) = str.ID(ele(iele).(eletype).enode,6);
    end
end
%------------------------------------------------
% Element coordinate
switch str.ndim
    case 2
        for iele = 1:str.nele
            eletype = ele(iele).type;
            str.elecoord(iele,1) = str.nodcoord(ele(iele).(eletype).snode,1);
            str.elecoord(iele,2) = str.nodcoord(ele(iele).(eletype).snode,2);
            str.elecoord(iele,3) = str.nodcoord(ele(iele).(eletype).enode,1);
            str.elecoord(iele,4) = str.nodcoord(ele(iele).(eletype).enode,2);
        end
    case 3
        for iele = 1:str.nele
            eletype = ele(iele).type;
            str.elecoord(iele,1) = str.nodcoord(ele(iele).(eletype).snode,1);
            str.elecoord(iele,2) = str.nodcoord(ele(iele).(eletype).snode,2);
            str.elecoord(iele,3) = str.nodcoord(ele(iele).(eletype).snode,3);
            str.elecoord(iele,4) = str.nodcoord(ele(iele).(eletype).enode,1);
            str.elecoord(iele,5) = str.nodcoord(ele(iele).(eletype).enode,2);
            str.elecoord(iele,6) = str.nodcoord(ele(iele).(eletype).enode,3);
        end
end
%------------------------------------------------
% Element lengths and direction cosines
switch str.ndim
    case 2
        for iele = 1:str.nele
            eletype = ele(iele).type;
            xs      = str.elecoord(iele,1);
            ys      = str.elecoord(iele,2);
            xe      = str.elecoord(iele,3);
            ye      = str.elecoord(iele,4);
            dx      = xe - xs;
            dy      = ye - ys;
            ele(iele).(eletype).L  = sqrt( dx*dx + dy*dy );
            ele(iele).(eletype).Cx = dx/ele(iele).(eletype).L;
            ele(iele).(eletype).Cy = dy/ele(iele).(eletype).L;
        end
    case 3
        for iele = 1:str.nele
            eletype = ele(iele).type;
            xs      = str.elecoord(iele,1);
            ys      = str.elecoord(iele,2);
            zs      = str.elecoord(iele,3);
            xe      = str.elecoord(iele,4);
            ye      = str.elecoord(iele,5);
            ze      = str.elecoord(iele,6);
            dx      = xe - xs;
            dy      = ye - ys;
            dz      = ze - zs;
            ele(iele).(eletype).L  = sqrt( dx*dx + dy*dy + dz*dz );
            ele(iele).(eletype).Cx = dx/ele(iele).(eletype).L;
            ele(iele).(eletype).Cy = dy/ele(iele).(eletype).L;
            ele(iele).(eletype).Cz = dz/ele(iele).(eletype).L;
        end
end
%------------------------------------------------
% Transformation matrix
for iele = 1:str.nele
    eletype = ele(iele).type;
    Cx = ele(iele).(eletype).Cx;
    Cy = ele(iele).(eletype).Cy;
    switch eletype
        case 'Simple2DTruss'
            [Gamma] = Simple2DTrussTransformationMarix(Cx, Cy);
        case 'Simple3DTruss'
            Cz = ele(iele).(eletype).Cz;
            [Gamma] = Simple3DTrussTransformationMarix(Cx, Cy, Cz);
        case 'StiffnessBasedBeam'
            [Gamma] = StiffnessBasedBeamTransformationMarix;
        case 'StiffnessBased2DBeamColumn'
            [Gamma] = StiffnessBased2DBeamColumnTransformationMarix(Cx, Cy);
        case 'StiffnessBased3DBeamColumn'
            Cz     = ele(iele).(eletype).Cz;
            alphax = ele(iele).(eletype).alphax;
            [Gamma] = StiffnessBased3DBeamColumnTransformationMarix(Cx, Cy, Cz, alphax);
        case 'Grid'
            Cz = ele(iele).(eletype).Cz;
            [Gamma] = GridTransformationMarix(Cx, Cy);
    end
    ele(iele).(eletype).Gamma = Gamma;
end