%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: FEAnalysis.m
%==========================================================

function [str, ele] = FEAnalysis(str, ele, sec, mat)
%------------------------------------------------
% get initial element stiffness matrix
for iele = 1:str.nele
    eletype = ele(iele).type;
    tmpele = ele(iele).(eletype);
    switch eletype
        case 'Simple2DTruss'
            [inistiff] = Simple2DTrussInitialStiffnessMatrix(tmpele, sec, mat);
        case 'Simple3DTruss'
            [inistiff] = Simple3DTrussInitialStiffnessMatrix(tmpele, sec, mat);
        case 'StiffnessBasedBeam'
            [inistiff] = StiffnessBasedBeamInitialStiffnessMatrix(tmpele, sec, mat);
        case 'StiffnessBased2DBeamColumn'
            [inistiff] = StiffnessBased2DBeamColumnInitialStiffnessMatrix(tmpele, sec, mat);
        case 'StiffnessBased3DBeamColumn'
            [inistiff] = StiffnessBased3DBeamColumnInitialStiffnessMatrix(tmpele, sec, mat);
        case 'Grid'
            [inistiff] = GridInitialStiffnessMatrix(tmpele, sec, mat);
    end
    ele(iele).(eletype).inibarke = inistiff.barke;
    ele(iele).(eletype).iniKe    = inistiff.Ke;
end
%------------------------------------------------
% get initial structural stiffness matrix
for k = 1:str.nele
    eletype = ele(iele).type;
    for i = 1:str.ndofpe
        for j = 1:str.ndofpe
            iniKe(i,j,k) = ele(k).(eletype).iniKe(i,j);
        end
    end
end
[iniKtt, iniKtu, iniKut, iniKuu, iniKS] = MatrixAssemble(str, iniKe);
str.iniKtt(:,:) = iniKtt(:,:);
str.iniKtu(:,:) = iniKtu(:,:);
str.iniKut(:,:) = iniKut(:,:);
str.iniKuu(:,:) = iniKuu(:,:);
str.iniKS(:,:)  = iniKS(:,:);
%------------------------------------------------
fprintf('----------------------\n')
fprintf('Start: Static analysis\n');
%------------------------------------------------
[str,ele] = staticanalysis(str, ele, sec, mat);
%------------------------------------------------
fprintf('----------------------\n')
fprintf('End  : Static analysis\n');
fprintf('----------------------\n')

%================================================
% str: Structure with information on structural level
% ele: Structure with information on element level
% sec: Structure with information on section level
% mat: Structure with information on material level
% eletype: Element type
% inistiff: Structure with initial stiffness matrices(barke, ke)
% Simple2DTrussInitialStiffnessMatrix: function for determination of
%   initial stiffness matrix of simple 2d truss
% Simple3DTrussInitialStiffnessMatrix: function for determination of
%   initial stiffness matrix of simple 3d truss 
% StiffnessBasedBeamInitialStiffnessMatrix: function for determination of
%   initial stiffness matrix of stiffness-based beams
% StiffnessBased2DBeamColumnInitialStiffnessMatrix: function for determination of
%   initial stiffness matrix of stiffness-based 2d beam-column
% StiffnessBased3DBeamColumnInitialStiffnessMatrix: function for determination of
%   initial stiffness matrix of stiffness-based 3d beam-column
% GridInitialStiffnessMatrix: function for determination of
%   initial stiffness matrix of grid
% ele(iele).(eletype).inibarke = inistiff.barke: initial element stiffness
%    matrix on local reference
% ele(iele).(eletype).iniKe = inistiff.Ke: initial element stiffness
%    matrix on global reference
% str.nele: Number of elements in str structure
% str.ndofpe: Number of d.o.fs per element in str structure
% MatrixAssemble: function for matrix assemble
%   (from element level to structure level)
% str.iniKtt = iniKtt: Stiffness matrix on initial Ktt
% str.iniKtu = iniKtu: Stiffness matrix on initial Ktu
% str.iniKut = iniKut: Stiffness matrix on initial Kut
% str.iniKuu = iniKuu: Stiffness matrix on initial Kuu 
% str.iniKS  = iniKS: Initial augmented stiffness matrix in structure level
%================================================

