%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: GridInitialStiffnessMatrix.m
%==========================================================

function [ini] = GridInitialStiffnessMatrix(ele, sec, mat)
%------------------------------------------------
sectag = ele.sectag;
L      = ele.L;
Gamma  = ele.Gamma;
switch sec(sectag).type
    case 'General'
        mattag = sec(sectag).mattag;
        E      = mat(mattag).E;
        G      = mat(mattag).G;
        Ix     = sec(sectag).Iy;
        Iz     = sec(sectag).Iz;
        GIx    = G * Ix;        
        EIz    = E * Iz;
end
%----------------------------------------------------------
ini.barke(:,:) = [ GIx/L,           0,          0, -GIx/L,           0,          0;
                       0,  12*EIz/L^3,  6*EIz/L^2,      0, -12*EIz/L^3,  6*EIz/L^2;
                       0,   6*EIz/L^2,    4*EIz/L,      0,  -6*EIz/L^2,    2*EIz/L;
                  -GIx/L,           0,          0,  GIx/L,           0,          0;
                       0, -12*EIz/L^3, -6*EIz/L^2,      0,  12*EIz/L^3, -6*EIz/L^2;
                       0,   6*EIz/L^2,    2*EIz/L,      0,  -6*EIz/L^2,    4*EIz/L] ;
%----------------------------------------------------------
ini.Ke(:,:) = Gamma(:,:)' * ini.barke * Gamma(:,:);
%==========================================================
% sectag: Section tag of element
% L: Element length
% Gamma: Transformation matrix from global to local
% sec(sectag).type: Section type
% mattag: Material tag of section
% E: Young's modulus
% G: Shear modulus
% Ix: Moment inertia of x-axis
% Iz: Moment inertia of y-axis
% GIx = G*Ix
% EIz = E*Iz
% ini.barke: Initial element stiffness matrix on local reference
% ini.Ke: Initial element stiffness matrix on global reference
%==========================================================