%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: NodeInfo.m
%==========================================================

function [str] = NodeInfo(str, nodcoord, constraint)
%------------------------------------------------
str.nnode = size(nodcoord,1);
%------------------------------------------------
% Node coordinate
for inode = 1:str.nnode
    nodetag = nodcoord{inode,1};
    switch str.ndim
        case 2
            str.nodcoord(nodetag, 1) = nodcoord{inode, 2};
            str.nodcoord(nodetag, 2) = nodcoord{inode, 3};
        case 3
            str.nodcoord(nodetag, 1) = nodcoord{inode, 2};
            str.nodcoord(nodetag, 2) = nodcoord{inode, 3};
            str.nodcoord(nodetag, 3) = nodcoord{inode, 4};
    end
end
%------------------------------------------------
% Make original ID matrix
str.originalID(1:str.nnode, 1:str.ndofpn) = 0;
ntemp = size(constraint,1);
for itemp = 1:ntemp
    nodetag = constraint{itemp,1};
    for idofpn=1:str.ndofpn
        str.originalID(nodetag,idofpn) = constraint{itemp,idofpn+1};
    end
end
%------------------------------------------------
% *** Make ID matrix ***
% Free d.o.f's
str.ID(1:str.nnode, 1:str.ndofpn) = 0;
nID = 1;
for inode = 1:str.nnode
    for idofpn = 1:str.ndofpn
        if( str.originalID(inode,idofpn) == 0 )
            str.ID(inode,idofpn) = nID;
            nID = nID+1;
        end
    end
end
str.nfdofs = nID - 1;
if (str.nfdofs == 0)
    error('Error: nfdofs must not be zero, that is, it has at least one.');
end
% Constraint d.o.f's
for inode = 1:str.nnode
    for idofpn = 1:str.ndofpn
        if( str.originalID(inode,idofpn) == 1 )
            str.ID(inode,idofpn) = nID;
            nID = nID+1;
        end
    end
end
str.ntdofs = nID - 1;
str.ncdofs = str.ntdofs - str.nfdofs;
%==========================================================
% Description of variables
%----------------------------------------------------------
% str       : Structure with information on structural level
% nodcoord  : Information on node coordinates
% constraint: Information on boundary condition
% str.nnode : Number of nodes
% str.ndim  : Definition of dimensions
% str.ndofpn: Number of degrees of freedom
% str.nodcoord: Storage with information on node coordinates
% nodetag   : Store node tag temporarily
% nID       : Counter of ID
% str.nfdofs: Number of free degrees of freedom
% str.ntdofs: Number of total degees of freedom
% str.ncdofs: Number of constraint degrees of freedom
% str.originalID(i,j): Original ID matrix(ith node, jth direction)
%       < Describe boundary condition and be made up of 0 or 1. >
% str.ID(i,j)        : ID matrix(ith node, jth direction)
%       < Describe boundary condition and be made up of positive integer
%       from 1 to str.ntdofs >
%==========================================================