%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: StaticForceInfo.m
%==========================================================

function [str, ele] = StaticForceInfo(str, ele, staticforces)
%------------------------------------------------
% Initialze external static forces
% Nodal force
str.static_Pt(1:str.nfdofs,1) = 0;
% Settlement
str.static_uu(1:str.ncdofs,1) = 0;
% Initialize nodal equivalent forces due to element distributed force on local and global reference.
for iele = 1:str.nele
    eletype = ele(iele).type;
    ele(iele).(eletype).static_local_nef(1:str.ndofpe)  = 0;
    ele(iele).(eletype).static_global_nef(1:str.ndofpe) = 0;
end
%------------------------------------------------
str.nstatic = size(staticforces, 1);
for iforce = 1:str.nstatic
    switch staticforces{iforce,1}
        case 'NodalForce'
            nfdofs = str.nfdofs;
            ID     = str.ID;
            [static_nf] = nodalforce(nfdofs, ID, iforce, staticforces);
            str.static_nf = static_nf;
            str.static_Pt(:,1) = str.static_Pt(:,1) + str.static_nf(:,1);
        case 'Settlement'
            nfdofs = str.nfdofs;
            ncdofs = str.ncdofs;
            ID     = str.ID;
            [static_settle] = settlement(nfdofs, ncdofs, ID, iforce, staticforces);
            str.static_settle = static_settle;
            str.static_uu(:,1) =  str.static_uu(:,1) + str.static_settle(:,1);
        case 'ElementDistributedForce'
            ndofpe = str.ndofpe;
            nfdofs = str.nfdofs;
            nele   = str.nele;
            LM     = str.LM;
            [ele, static_edf2nef] = elementdistributedforce(ndofpe, nfdofs, nele, LM, ele, iforce, staticforces);
            str.static_edf2nef = static_edf2nef;
            str.static_Pt(:,1) = str.static_Pt(:,1) + str.static_edf2nef(:,1);
    end
end