%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: StiffnessBased3DBeamColumnInitialStiffnessMatrix.m
%==========================================================

function [ini] = StiffnessBased3DBeamColumnInitialStiffnessMatrix(ele, sec, mat)
%------------------------------------------------
sectag = ele.sectag;
L      = ele.L;
Gamma  = ele.Gamma;
switch sec(sectag).type
    case 'General'
        mattag = sec(sectag).mattag;
        E      = mat(mattag).E;
        G      = mat(mattag).G;
        A      = sec(sectag).A;
        Ix     = sec(sectag).Ix;
        Iy     = sec(sectag).Iy;
        Iz     = sec(sectag).Iz;
        EA     = E * A;
        GIx    = G * Ix;
        EIy    = E * Iy;
        EIz    = E * Iz;
end
%----------------------------------------------------------
% Set sub matrix k1, k2, k3, and k4
k1= [EA/L,          0,          0;
        0, 12*EIz/L^3,          0;
        0,          0, 12*EIy/L^3];
                    
k2 = [0,          0,         0;
      0,          0, 6*EIz/L^2;
      0, -6*EIy/L^2,         0];
                    
k3 = [GIx/L,       0,       0;
          0, 4*EIy/L,       0;
          0,       0, 4*EIz/L];
                                                      
k4 = [-GIx/L,       0,       0;
           0, 2*EIy/L,       0;
           0,       0, 2*EIz/L];
%----------------------------------------------------------                      
ini.barke(:,:) = [  k1,  k2,  -k1,  k2;                  
                   k2',  k3, -k2',  k4;                  
                   -k1, -k2,   k1, -k2; 
                   k2',  k4, -k2',  k3];  
%----------------------------------------------------------
ini.Ke(:,:) = Gamma(:,:)' * ini.barke * Gamma(:,:);
%==========================================================
% sectag: Section tag of element
% L: Element length
% Gamma: Transformation matrix from global to local
% sec(sectag).type: Section type
% mattag: Material tag of section
% E: Young's modulus
% G: Shear modulus
% A: Cross-sectional area
% Ix: Moment inertia of x-axis
% Iy: Moment inertia of y-axis
% Iz: Moment inertia of z-axis
% EA + E*A
% GIx = G*Ix
% EIy = E*Iy
% EIz = E*Iz
% k1, k2, k3, k4: Sub-matrices in barke
% ini.barke: Initial element stiffness matrix on local reference
% ini.Ke: Initial element stiffness matrix on global reference
%==========================================================