%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: StiffnessBasedBeamInitialStiffnessMatrix.m
%==========================================================

function [ini] = StiffnessBasedBeamInitialStiffnessMatrix(ele, sec, mat)
%------------------------------------------------
sectag = ele.sectag;
L      = ele.L;
Gamma  = ele.Gamma;
switch sec(sectag).type
    case 'General'
        mattag = sec(sectag).mattag;
        E      = mat(mattag).E;
        Iz     = sec(sectag).Iz;
        EIz    = E * Iz;
end
%----------------------------------------------------------
ini.barke(:,:) = [ 12*EIz/L^3,  6*EIz/L^2, -12*EIz/L^3,  6*EIz/L^2;
                    6*EIz/L^2,    4*EIz/L,  -6*EIz/L^2,    2*EIz/L;
                  -12*EIz/L^3, -6*EIz/L^2,  12*EIz/L^3, -6*EIz/L^2;
                    6*EIz/L^2,    2*EIz/L,  -6*EIz/L^2,    4*EIz/L];
%----------------------------------------------------------
ini.Ke(:,:) = Gamma(:,:)' * ini.barke * Gamma(:,:);
%==========================================================
% sectag: Section tag of element
% L: Element length
% Gamma: Transformation matrix from global to local
% sec(sectag).type: Section type
% mattag: Material tag of section
% E: Young's modulus
% Iz: Moment inertia of y-axis
% EIz = E*Iz
% ini.barke: Initial element stiffness matrix on local reference
% ini.Ke: Initial element stiffness matrix on global reference
%==========================================================