%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: WriteOutputFile.m
%==========================================================

function WriteOutputFile(str, ele, sec, mat, staticforces)
%------------------------------------------------
filename = input('Output file name[*.out]: ','s');
fid = fopen(filename, 'w', 'n');
%------------------------------------------------
fprintf(fid,'********** Input file name: %s **********\n', str.infile);
fprintf(fid,'Force unit : %s\n', str.Funit);
fprintf(fid,'Length unit: %s\n', str.Lunit);
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Number of nodes      : %d\n', str.nnode);
fprintf(fid,'Number of elements   : %d\n', str.nele);
fprintf(fid,'Number of sections   : %d\n', str.nsec);
fprintf(fid,'Number of materials  : %d\n', str.nmat);
fprintf(fid,'Number of total d.o.fs      : %d\n', str.ntdofs);
fprintf(fid,'Number of free d.o.fs       : %d\n', str.nfdofs);
fprintf(fid,'Number of constranied d.o.fs: %d\n', str.ncdofs);
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Node information:\n');
switch str.ndim
    case 2
    for inode = 1:str.nnode
            fprintf(fid,'Node %3d: x = %10.4f, y = %10.4f\n',inode, str.nodcoord(inode, 1), str.nodcoord(inode, 2));
        end
    case 3
        for inode = 1:str.nnode
            fprintf(fid,'Node %3d: x = %10.4f, y = %10.4f, z = %10.4f\n',inode, str.nodcoord(inode, 1), str.nodcoord(inode, 2), str.nodcoord(inode, 3));
        end
end
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Boundary condition (0: free, 1: constraint):\n');
switch str.ndofpn
    case 2
        for inode = 1:str.nnode
            fprintf(fid,'Node %3d: Id1 = %d, Id2 = %d\n',inode, str.originalID(inode, 1), str.originalID(inode, 2));
        end
    case 3
        for inode = 1:str.nnode
            fprintf(fid,'Node %3d: Id1 = %d, Id2 = %d\n, Id3 = %d\n',inode, str.originalID(inode, 1), str.originalID(inode, 2), str.originalID(inode, 3));
        end
    case 6
        for inode = 1:str.nnode
            fprintf(fid,'Node %3d: Id1 = %d, Id2 = %d\n, Id3 = %d, Id4 = %d, Id5 = %d\n, Id6 = %d\n',...
                inode, str.originalID(inode, 1), str.originalID(inode, 2), str.originalID(inode, 3),...
                str.originalID(inode, 4), str.originalID(inode, 5), str.originalID(inode, 6));
        end
end
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Element information:\n');
for iele = 1:str.nele
    eletype = ele(iele).type;
    snode  = ele(iele).(eletype).snode;
    enode  = ele(iele).(eletype).enode;
    sectag = ele(iele).(eletype).sectag;
    switch eletype
        case 'StiffnessBased3DBeamColumn'
            alphax = ele(iele).(eletype).alphax;
            fprintf(fid,'Element %3d (%3d -> %3d): Type - %s with rotation of x-axis(%7.4f radian) and section %3d\n',...
                iele, snode, enode, eletype, alphax, sectag);
        otherwise
            fprintf(fid,'Element %3d (%3d -> %3d): Type - %s with section %3d\n', iele, snode, enode, eletype, sectag);
    end
end
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Section information:\n');
for isec = 1:str.nsec
    fprintf(fid,'Section %3d: Type - %s, A = %12.5e, Ix = %12.5e, Iy = %12.5e, Iz = %12.5e with material %3d\n',...
        isec, sec(isec).type, sec(isec).A, sec(isec).Ix, sec(isec).Iy, sec(isec).Iz, sec(isec).mattag);
end
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Material information:\n');
for imat = 1:str.nmat
    fprintf(fid,'Material %3d: Type - %s, E = %12.5e, G = %12.5e\n',...
        imat, mat(imat).type, mat(imat).E, mat(imat).G);
end
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Force information:\n');
str.nstatic = size(staticforces, 1);
for iforce = 1:str.nstatic
    switch staticforces{iforce,1}
        case 'NodalForce'
            nfsize = size(staticforces{iforce,2},1);
            unit    = [str.Funit,' or ',str.Funit,'.',str.Lunit];
            for ifsize = 1:nfsize
                nodenum = staticforces{iforce,2}{ifsize,1};
                dof     = staticforces{iforce,2}{ifsize,2};
                mag     = staticforces{iforce,2}{ifsize,3};
                fprintf(fid,'Node %3d: Global direction(%d), Magnitude(%f [%s])\n', nodenum, dof, mag, unit);
            end
        case 'Settlement'
            nssize = size(staticforces{iforce,2},1);
            unit    = [str.Lunit,' or ','radian'];
            for issize = 1:nssize
                nodenum = staticforces{iforce,2}{issize,1};
                dof     = staticforces{iforce,2}{issize,2};
                mag     = staticforces{iforce,2}{issize,3};
                fprintf(fid,'Node %3d: Global direction(%d), Displacement(%f [%s])\n', nodenum, dof, mag, unit);
            end
        case 'ElementDistributedForce'
            nedfsize = size(staticforces{iforce,2},1);
            unit    = [str.Funit,' or ',str.Funit,'.',str.Lunit];
            for iedfsize = 1:nedfsize
                elenum = staticforces{iforce,2}{iedfsize,1};
                dof     = staticforces{iforce,2}{iedfsize,2};
                mag     = staticforces{iforce,2}{iedfsize,3};
                fprintf(fid,'Element %3d: Local direction(%d), Magnitude(%f [%s]\n)', elenum, dof, mag, unit);
            end
    end
end
fprintf(fid,'********** Output file name: %s **********\n', filename);

fprintf(fid,'Element nodal displacements:\n');
for iele = 1:str.nele
    eletype = ele(iele).type;
    tmpele  = ele(iele).(eletype);
    switch eletype
        case 'Simple2DTruss'
            fprintf(fid,'Element %3d (%s):\n', iele, eletype);
            fprintf(fid,'        Start node %3d: ux(%f %s)\n', tmpele.snode, tmpele.barde(1,1), str.Lunit); 
            fprintf(fid,'        End   node %3d: ux(%f %s)\n', tmpele.enode, tmpele.barde(2,1), str.Lunit); 
        case 'Simple3DTruss'
            fprintf(fid,'Element %3d (%s):', iele, eletype);
            fprintf(fid,'        Start node %3d: ux(%f %s)\n', tmpele.snode, tmpele.barde(1,1), str.Lunit); 
            fprintf(fid,'        End   node %3d: ux(%f %s)\n', tmpele.enode, tmpele.barde(2,1), str.Lunit); 
        case 'StiffnessBasedBeam'
            fprintf(fid,'Element %3d (%s):\n', iele, eletype);
            angle = 'radian';
            fprintf(fid,'        Start node %3d: uy(%f %s), theta_z(%f %s)\n', tmpele.snode, tmpele.barde(1,1), str.Lunit, tmpele.barde(2,1), angle); 
            fprintf(fid,'        End   node %3d: uy(%f %s), theta_z(%f %s)\n', tmpele.enode, tmpele.barde(3,1), str.Lunit, tmpele.barde(4,1), angle); 
        case 'StiffnessBased2DBeamColumn'
            fprintf(fid,'Element %3d (%s):', iele, eletype);
            angle = 'radian';
            fprintf(fid,'        Start node %3d: ux(%f %s), uy(%f %s), theta_z(%f %s)\n', tmpele.snode, tmpele.barde(1,1), str.Lunit, tmpele.barde(2,1), str.Lunit, tmpele.barde(3,1), angle);
            fprintf(fid,'        End   node %3d: ux(%f %s), uy(%f %s), theta_z(%f %s)\n', tmpele.enode, tmpele.barde(4,1), str.Lunit, tmpele.barde(5,1), str.Lunit, tmpele.barde(6,1), angle);
        case 'StiffnessBased3DBeamColumn'
            fprintf(fid,'Element %3d (%s):', iele, eletype);
            angle = 'radian';
            fprintf(fid,'        Start node %3d: ux(%f %s), uy(%f %s), uz(%f %s), theta_x(%f %s), theta_y(%f %s), theta_z(%f %s)\n',...
                tmpele.snode, tmpele.barde(1,1), str.Lunit, tmpele.barde(2,1), str.Lunit, tmpele.barde(3,1), str.Lunit, tmpele.barde(4,1), angle, tmpele.barde(5,1), angle, tmpele.barde(6,1), angle);
            fprintf(fid,'        Start node %3d: ux(%f %s), uy(%f %s), uz(%f %s), theta_x(%f %s), theta_y(%f %s), theta_z(%f %s)\n',...
                tmpele.enode, tmpele.barde(7,1), str.Lunit, tmpele.barde(8,1), str.Lunit, tmpele.barde(9,1), str.Lunit, tmpele.barde(10,1), angle, tmpele.barde(11,1), angle, tmpele.barde(12,1), angle);
        case 'Grid'
            fprintf(fid,'Element %3d (%s):', iele, eletype);
            angle = 'radian';
            fprintf(fid,'        Start node %3d: theta_x(%f %s), uy(%f %s), theta_z(%f %s)\n', tmpele.snode, tmpele.barde(1,1), angle, tmpele.barde(2,1), str.Lunit, tmpele.barde(3,1), angle);
            fprintf(fid,'        End   node %3d: theta_x(%f %s), uy(%f %s), theta_z(%f %s)\n', tmpele.enode, tmpele.barde(4,1), angle, tmpele.barde(5,1), str.Lunit, tmpele.barde(6,1), angle);
    end
end
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Element nodal forces(Element interanl nodal forces):\n');
for iele = 1:str.nele
    eletype = ele(iele).type;
    tmpele  = ele(iele).(eletype);
    switch eletype
        case 'Simple2DTruss'
            fprintf(fid,'Element %3d (%s):\n', iele, eletype);
            fprintf(fid,'        Start node %3d: barfx(%f %s)\n', tmpele.snode, tmpele.intbarfe(1,1), str.Funit); 
            fprintf(fid,'        End   node %3d: barfx(%f %s)\n', tmpele.enode, tmpele.intbarfe(2,1), str.Funit); 
        case 'Simple3DTruss'
            fprintf(fid,'Element %3d (%s):', iele, eletype);
            fprintf(fid,'        Start node %3d: barfx(%f %s)\n', tmpele.snode, tmpele.intbarfe(1,1), str.Funit); 
            fprintf(fid,'        End   node %3d: barfx(%f %s)\n', tmpele.enode, tmpele.intbarfe(2,1), str.Funit); 
        case 'StiffnessBasedBeam'
            fprintf(fid,'Element %3d (%s):\n', iele, eletype);
            munit = [str.Funit,'.',str.Lunit];
            fprintf(fid,'        Start node %3d: barfy(%f %s), barMz(%f %s)\n', tmpele.snode, tmpele.intbarfe(1,1), str.Funit, tmpele.intbarfe(2,1), munit); 
            fprintf(fid,'        End   node %3d: barfy(%f %s), barMz(%f %s)\n', tmpele.enode, tmpele.intbarfe(3,1), str.Funit, tmpele.intbarfe(4,1), munit); 
        case 'StiffnessBased2DBeamColumn'
            fprintf(fid,'Element %3d (%s):', iele, eletype);
            munit = [str.Funit,'.',str.Lunit];
            fprintf(fid,'        Start node %3d: barfx(%f %s), barfy(%f %s), barMz(%f %s)\n', tmpele.snode, tmpele.intbarfe(1,1), str.Funit, tmpele.intbarfe(2,1), str.Funit, tmpele.intbarfe(3,1), munit);
            fprintf(fid,'        End   node %3d: barfx(%f %s), barfy(%f %s), barMz(%f %s)\n', tmpele.enode, tmpele.intbarfe(4,1), str.Funit, tmpele.intbarfe(5,1), str.Funit, tmpele.intbarfe(6,1), munit);
        case 'StiffnessBased3DBeamColumn'
            fprintf(fid,'Element %3d (%s):', iele, eletype);
            munit = [str.Funit,'.',str.Lunit];
            fprintf(fid,'        Start node %3d: barfx(%f %s), barfy(%f %s), barfz(%f %s), barMx(%f %s), barMy(%f %s), barMz(%f %s)\n',...
                tmpele.snode, tmpele.intbarfe(1,1), str.Funit, tmpele.intbarfe(2,1), str.Funit, tmpele.intbarfe(3,1), str.Funit, tmpele.intbarfe(4,1), munit, tmpele.intbarfe(5,1), munit, tmpele.intbarfe(6,1), munit);
            fprintf(fid,'        End   node %3d: barfx(%f %s), barfy(%f %s), barfz(%f %s), barMx(%f %s), barMy(%f %s), barMz(%f %s)\n',...
                tmpele.snode, tmpele.intbarfe(7,1), str.Funit, tmpele.intbarfe(8,1), str.Funit, tmpele.intbarfe(9,1), str.Funit, tmpele.intbarfe(10,1), munit, tmpele.intbarfe(11,1), munit, tmpele.intbarfe(12,1), munit);
        case 'Grid'
            fprintf(fid,'Element %3d (%s):', iele, eletype);
            munit = [str.Funit,'.',str.Lunit];
            fprintf(fid,'        Start node %3d: barMx(%f %s), barfy(%f %s), barMz(%f %s)\n', tmpele.snode, tmpele.intbarfe(1,1), munit, tmpele.intbarfe(2,1), str.Funit, tmpele.intbarfe(3,1), munit);
            fprintf(fid,'        End   node %3d: barMx(%f %s), barfy(%f %s), barMz(%f %s)\n', tmpele.enode, tmpele.intbarfe(4,1), munit, tmpele.intbarfe(5,1), str.Funit, tmpele.intbarfe(6,1), munit);
    end
end
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Nodal displacements:\n');
ID = str.ID;
uu = str.static_uu;
ut = str.ut;
nfdofs = str.nfdofs;
ndofpn = str.ndofpn;
for inode = 1:str.nnode
    switch ndofpn
        case 2
            for idofpn = 1:ndofpn
                if ID(inode,idofpn) <= nfdofs
                    U(idofpn) = ut(ID(inode,idofpn),1);
                else
                    U(idofpn) = uu(ID(inode,idofpn)-nfdofs,1);
                end
            end
            unit1 = [str.Lunit];
            unit2 = [str.Lunit,' or ','radian'];
            fprintf(fid,'Node %3d: U1(%f %s), U2(%f [%s])\n', inode, U(1), unit1, U(2), unit2);
        case 3
            for idofpn = 1:ndofpn
                if ID(inode,idofpn) <= nfdofs
                    U(idofpn) = ut(ID(inode,idofpn),1);
                else
                    U(idofpn) = uu(ID(inode,idofpn)-nfdofs,1);
                end
            end
            unit1 = [str.Lunit];
            unit2 = [str.Lunit,' or ','radian'];
            fprintf(fid,'Node %3d: U1(%f %s), U2(%f %s), U3(%f [%s])\n', inode, U(1), unit1, U(2), unit1, U(3), unit2);
        case 6
            for idofpn = 1:ndofpn
                if ID(inode,idofpn) <= nfdofs
                    U(idofpn) = ut(ID(inode,idofpn),1);
                else
                    U(idofpn) = uu(ID(inode,idofpn)-nfdofs,1);
                end
            end
            unit1 = [str.Lunit];
            unit2 = 'radian';
            fprintf(fid,'Node %3d: U1(%f %s), U2(%f %s), U3(%f %s), U1(%f %s), U2(%f %s), U3(%f %s)\n', inode, U(1), unit1, U(2), unit1, U(3), unit1, U(4), unit2, U(5), unit2, U(6), unit2);
    end
end
fprintf(fid,'------------------------------------------------\n');
fprintf(fid,'Reactions:\n');
react = str.reaction;
for inode = 1:str.nnode
    unit = [str.Funit,' or ', str.Funit,'.', str.Lunit];
    switch ndofpn
        case 2
            for idofpn = 1:ndofpn
                if ID(inode,idofpn) > nfdofs
                    fprintf(fid,'Node %3d: F%d(%f [%s])\n', inode, idofpn, react(ID(inode,idofpn)-nfdofs), unit); 
                end
            end
        case 3
            for idofpn = 1:ndofpn
                if ID(inode,idofpn) > nfdofs
                    fprintf(fid,'Node %3d: F%d(%f [%s])\n', inode, idofpn, react(ID(inode,idofpn)-nfdofs), unit); 
                end
            end
        case 6
            for idofpn = 1:ndofpn
                if ID(inode,idofpn) > nfdofs
                    fprintf(fid,'Node %3d: F%d(%f [%s])\n', inode, idofpn, react(ID(inode,idofpn)-nfdofs), unit); 
                end
            end
    end
end
fprintf(fid,'********** End: FE analysis of %s **********\n', str.infile);
%------------------------------------------------
fclose(fid);