%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: elementdistributedforce.m
%==========================================================

function [ele, static_edf2nef] = elementdistributedforce(ndofpe, nfdofs, nele, LM, ele, iforce, staticforces)

%----------------------------------------------------------
nedf = size(staticforces{iforce,2},1);   % number of element distributed forces
%----------------------------------------------------------
for iedf = 1:nedf
    eletag  = staticforces{iforce,2}{iedf,1};    % element tag
    eletype = ele(eletag).type;                  % element type
    L       = ele(eletag).(eletype).L;                     % element length
    m       = staticforces{iforce,2}{iedf,3};    % magnitude on element distributed force
    switch eletype
        case 'StiffnessBasedBeam'
            if(staticforces{iforce,2}{iedf,2} == 1)             % y direction
                ele(eletag).(eletype).edf_ydir            =  m;           % element distributed force of y direction
                ele(eletag).(eletype).static_local_nef(1) =  m * L/2;     % y direction of start node
                ele(eletag).(eletype).static_local_nef(2) =  m * L^2/12;  % tz direction of start node
                ele(eletag).(eletype).static_local_nef(3) =  m * L/2;     % y direction of end node
                ele(eletag).(eletype).static_local_nef(4) = -m * L^2/12;  % tz direction of end node
            elseif(staticforces{iforce,2}{iedf,2} == 2)         % tz direction
                error('Input error: Not support moment of tz direction as external element distributed force');
            end
            ele(eletag).(eletype).static_global_nef(:)  = ele(eletag).(eletype).Gamma' * ele(eletag).(eletype).static_local_nef(:);
        case 'StiffnessBased2DBeamColumn'
            if(staticforces{iforce,2}{iedf,2} == 1)             % x direction
                ele(eletag).(eletype).edf_xdir            =  m;           % element distributed force of x direction
                ele(eletag).(eletype).static_local_nef(1) =  m * L/2;     % x direction of start node
                ele(eletag).(eletype).static_local_nef(4) =  m * L/2;     % x direction of end node
            elseif(staticforces{iforce,2}{iedf,2} == 2)         % y direction
                ele(eletag).(eletype).edf_ydir            =  m;           % element distributed force of y direction
                ele(eletag).(eletype).static_local_nef(2) =  m * L/2;     % y direction of start node
                ele(eletag).(eletype).static_local_nef(3) =  m * L^2/12;  % tz direction of start node
                ele(eletag).(eletype).static_local_nef(5) =  m * L/2;     % y direction of end node
                ele(eletag).(eletype).static_local_nef(6) = -m * L^2/12;  % tz direction of end node
            elseif(staticforces{iforce,2}{iedf,2} == 3)         % tz direction
                error('Input error: Not support moment of tz direction as external element distributed force');
            end
            ele(eletag).(eletype).static_global_nef(:)  = ele(eletag).(eletype).Gamma' * ele(eletag).(eletype).static_local_nef(:);
        case 'StiffnessBased3DBeamColumn'
            if(staticforces{iforce,2}{iedf,2} == 1)             % x direction
                ele(eletag).(eletype).edf_xdir            =  m;           % element distributed force of x direction
                ele(eletag).(eletype).static_local_nef(1) =  m * L/2;     % x direction of start node
                ele(eletag).(eletype).static_local_nef(7) =  m * L/2;     % x direction of end node
            elseif(staticforces{iforce,2}{iedf,2} == 2)         % y direction
                ele(eletag).(eletype).edf_ydir            =  m;           % element distributed force of y direction
                ele(eletag).(eletype).static_local_nef(2) =  m * L/2;     % y direction of start node
                ele(eletag).(eletype).static_local_nef(6) =  m * L^2/12;  % tz direction of start node
                ele(eletag).(eletype).static_local_nef(8) =  m * L/2;     % y direction end node
                ele(eletag).(eletype).static_local_nef(12)= -m * L^2/12;  % tz direction end node
            elseif(staticforces{iforce,2}{iedf,2} == 3)         % z direction
                ele(eletag).edf_zdir            =  m;           % element distributed force of z direction
                ele(eletag).(eletype).static_local_nef(3) =  m * L/2;     % z direction of start node
                ele(eletag).(eletype).static_local_nef(5) =  m * L^2/12;  % ty direction of start node
                ele(eletag).(eletype).static_local_nef(9) =  m * L/2;     % z direction of end node
                ele(eletag).(eletype).static_local_nef(11)= -m * L^2/12;  % ty direction of end node
            else
                error('Input error: Not support moment of tx, ty and tz direction  as external element distributed force');
            end
            ele(eletag).(eletype).static_global_nef(:)  = ele(eletag).(eletype).Gamma' * ele(eletag).(eletype).static_local_nef(:);
        case 'Grid'
            if(staticforces{iforce,2}{iedf,2} == 1)             % tx direction
                ele(eletag).(eletype).edf_txdir           =  m;           % element distributed force of tx direction
                ele(eletag).(eletype).static_local_nef(1) =  m * L/2;     % tx direction of start node
                ele(eletag).(eletype).static_local_nef(4) =  m * L/2;     % tx direction of end node
            elseif(staticforces{iforce,2}{iedf,2} == 2)         % y direction
                ele(eletag).(eletype).edf_ydir            =  m;           % element distributed force of y direction
                ele(eletag).(eletype).static_local_nef(2) =  m * L/2;     % y direction of start node
                ele(eletag).(eletype).static_local_nef(3) =  m * L^2/12;  % tz direction of start node
                ele(eletag).(eletype).static_local_nef(5) =  m * L/2;     % y direction of end node
                ele(eletag).(eletype).static_local_nef(6) = -m * L^2/12;  % tz direction of end node
            else
                error('Input error: Not support moment of tz direction as external element distributed force');
            end
            ele(eletag).(eletype).static_global_nef(:)  = ele(eletag).(eletype).Gamma' * ele(eletag).(eletype).static_local_nef(:);
    end
end
%----------------------------------------------------------
% Assemble nodal forces due to element distributed forces
% on global reference
static_edf2nef(1:nfdofs,1) = 0;
for iele = 1:nele
    eletype = ele(iele).type;
    for idofpe = 1:ndofpe
        temp = LM(iele,idofpe);
        if temp <= nfdofs
            static_edf2nef(temp,1) = static_edf2nef(temp,1) + ele(iele).(eletype).static_global_nef(idofpe);
        end
    end
end
%----------------------------------------------------------