%==========================================================
% Mercury Matlab Version 1.0 for Education
% Written by Dae-Hung Kang, CU-NEES
% Copyright 2009, CU-NEES
% Written  : September 2009.
% File name: staticlinear.m
%==========================================================

function [str, ele] = staticlinear(str, ele, sec, mat)
%------------------------------------------------
nfdofs = str.nfdofs;
ndofpe = str.ndofpe;
nele   = str.nele;
LM     = str.LM;
% Nodal displacement vector
str.ut(:,1) = inv(str.iniKtt)...
    * ( str.static_Pt(:,1) - str.iniKtu(:,:) * str.static_uu(:,1) );
%------------------------------------------------
% Element state determination
for iele = 1:nele
    eletype = ele(iele).type;
    %--------------------------------------------
    % Get element nodal displacement vector on global reference
    ele(iele).(eletype).deltae(1:ndofpe,1) = 0;
    for idofpe = 1:ndofpe
        tmp = LM(iele,idofpe);
        if ( tmp > nfdofs )     % For settlement
            ele(iele).(eletype).deltae(idofpe,1) = str.static_uu( LM(iele,idofpe) - nfdofs, 1 );
        elseif ( tmp <= nfdofs )
            ele(iele).(eletype).deltae(idofpe,1) = str.ut( LM(iele,idofpe) , 1 );
        end
    end
    %--------------------------------------------
    % Get element nodal displacement vector on local reference
    ele(iele).(eletype).barde(:,1) = ele(iele).(eletype).Gamma(:,:) * ele(iele).(eletype).deltae(:,1);
    %--------------------------------------------
    % Get internal element nodal force vector on local and global reference
    tmpintbarfe(:,1) = ele(iele).(eletype).inibarke(:,:) * ele(iele).(eletype).barde(:,1);
    switch eletype
        case 'Simple2DTruss'
            ele(iele).(eletype).intbarfe(:,1) = tmpintbarfe(:,1);
        case 'Simple3DTruss'
            ele(iele).(eletype).intbarfe(:,1) = tmpintbarfe(:,1);
        case 'StiffnessBasedBeam'
            ele(iele).(eletype).intbarfe(:,1) = tmpintbarfe(:,1) - ele(iele).(eletype).static_local_nef(:,1);
        case 'StiffnessBased2DBeamColumn'
            ele(iele).(eletype).intbarfe(:,1) = tmpintbarfe(:,1) - ele(iele).(eletype).static_local_nef(:,1);
        case 'StiffnessBased3DBeamColumn'
            ele(iele).(eletype).intbarfe(:,1) = tmpintbarfe(:,1) - ele(iele).(eletype).static_local_nef(:,1);
        case 'Grid'
            ele(iele).(eletype).intbarfe(:,1) = tmpintbarfe(:,1) - ele(iele).(eletype).static_local_nef(:,1);
    end
    ele(iele).(eletype).intFe = ele(iele).(eletype).Gamma'*ele(iele).(eletype).intbarfe(:,1);
end
%------------------------------------------------
% Get reaction
str.reaction(1:str.ncdofs,1) = 0;
for iele = 1:nele
    eletype = ele(iele).type;
    for idofpe = 1:ndofpe
        tmp = LM(iele,idofpe);
        if tmp > nfdofs
            str.reaction(tmp-nfdofs,1) = str.reaction(tmp-nfdofs,1) + ele(iele).(eletype).intFe(idofpe,1);
        end
    end
end
%------------------------------------------------